function tron = TRONCON(sys,Uj,Utj,diaginfos,params,docalculseries)

tron.ordre = get_ordre(sys);
tron.ninc = get_ninc(sys);

%% Initialisation of the object 'tron'
tron.dispcolors = diaginfos.dispcolors;
tron.dispvars = diaginfos.dispvars;
tron.seuil = get_seuil(sys);
tron.perturbation = diaginfos.perturbation;
nbptstroncon = get_nbptstroncon(sys);

tron.FpnlA = zeros(tron.ninc-1,1);
tron.Ups = zeros(tron.ninc,0);
tron.Amax = 1;

% Syntax : (nx2) matrix where n is the number of curves to plot
% Curves abscissa form the first column, and their ordinates 
% form the second
nbcourbes = length( diaginfos.dispvars(:,1));

tron.nbcourbes = nbcourbes;

tron.X = zeros(0, nbcourbes); % Abscissa axis parameter
tron.Y = zeros(0, nbcourbes); % Ordinate axis parameter
tron.A = zeros(0, nbcourbes); % A values for each (X,Y) point

tron.Utj = Utj; % Approx. tangent at starting point of the portion;
tron.Uj = Uj; % Starting point of the portion;

tron.drawtype = diaginfos.drawtype;

%% Stability-related fields initialisation
tron.drawstab = diaginfos.drawstab;
tron.stabvars = diaginfos.stabvars;
if params.stability ==  1
    tron.HH = diaginfos.stabvars(end);
    HH = tron.HH;
    nbddl = diaginfos.nbddl;
    nbddlstab = diaginfos.nbddlstab;
    if HH == 0
        if (length(diaginfos.stabvars) < 3)
            error('Vector ML_varstab too small.');
        end
        tron.NeqHH0 = tron.ninc - 1;
        tron.eigen = zeros(nbptstroncon,0);
        tron.vectors = zeros(nbptstroncon,0);
        tron.ddlstab = zeros(nbptstroncon,0);
    else
        if (length(diaginfos.stabvars) < 4)
            error('Vector ML_varstab too small..');
        end
        if diaginfos.stabvars(end-1) == diaginfos.stabvars(end-2);
            tron.NeqHH0 = (tron.ninc-1) / (2*(tron.HH)+1);
        else
            tron.NeqHH0 = (tron.ninc-2) / (2*(tron.HH)+1);
        end
        tron.eigen = zeros(nbptstroncon,0);
        tron.vectors = zeros(nbptstroncon,0);
        tron.ddlstab = zeros(nbptstroncon,0);
    end

    tron.nbddl=nbddl;
    tron.nbddlstab = nbddlstab;
    tron.stabcheck = zeros(nbptstroncon,1);   
else
    tron.HH = 0;
    tron.NeqHH0 = 0;
    tron.eigen = [];
    tron.vectors = [];
    tron.ddlstab = [];
    tron.nbddl=0;
    tron.nbddlstab = 0;
    tron.stabcheck = [];
end



%% Instanciation of the TRONCON class
tron = class(tron,'TRONCON');

%%  Computations
% test for real computation or initialisation only
if docalculseries ==1
    
    %%% Compute the power series of this portion
    [tron.Ups,tron.FpnlA] = calculeseries(sys,Uj,Utj);

    % Compute Amax
    if norm(tron.FpnlA) == 0
      Amax = 1;
      disp('/!\ WARNING /!\ The last term of the power series is null...');
      disp('Amax set to 1.');
    else
      Amax = (tron.seuil/norm(tron.FpnlA))^(1/(tron.ordre+1));
    end
    tron.Amax=Amax;

    %%% Stability analysis
    if params.stability ==  1
        [tron.ddlstab] = troncon_ddlstab(tron, tron.seuil, nbptstroncon);
        if params.algostab == 1
            [tron.stabcheck,tron.eigen, tron.vectors] = stabT(sys, tron.ddlstab, diaginfos, params, HH);
        else
            [tron.stabcheck,tron.eigen,tron.vectors] = stabF(sys, tron.ddlstab, diaginfos, params, HH);
        end
    end
end
