!     MEX-file for MATLAB function "L"
!     S. Karkar
!     October 2010

!     The calling arguments must be : (the_object_itself, U, H, Neq)

#include "fintrf.h"

subroutine mexFunction(nlhs, plhs, nrhs, prhs)

    use petits_operateurs
    implicit none
    integer nlhs, nrhs, H, Neq_in, N
    mwPointer plhs(nlhs), prhs(nrhs), L_pr, U_pr, H_pr, Neq_pr, mxGetPr, mxCreateDoubleMatrix
    double precision tmp

    if (nlhs /= 1) call mexErrMsgTxt('Error : there must be exactly 1 output argument.')
    if (nrhs /= 4) call mexErrMsgTxt('Error : there must be exactly 4 input arguments.')


!     get the harmonic number H and the equation number Neq
    H_pr = mxGetPr(prhs(3))
    call mxcopyptrtoreal8(H_pr,tmp,1)
    H=int(tmp)

    Neq_pr = mxGetPr(prhs(4))
    call mxcopyptrtoreal8(Neq_pr,tmp,1)
    Neq_in=int(tmp)

    if (Neq_in /= Neq) call mexerrmsgtxt('Neq not consistent with its value in Fortran sources!')

! size of the FORCED extended system
    N = Neq*(2*H+1) + 1

! get the U vector pointer
    U_pr = mxGetPr(prhs(2))

!     Create matrix for the return argument.
    plhs(1) = mxCreateDoubleMatrix(N-1,1,0)
    L_pr = mxgetpr(plhs(1))


!     Call the computational subroutine on L with U, H and N values
    call CALCUL_L(U_pr,H,N,L_pr)


    return
end subroutine mexFunction

subroutine CALCUL_L(U_pr,H,N,L_pr)
 
    use petits_operateurs
    integer(4) N,H,i,indc(H),inds(H)
    mwPointer U_pr, L_pr
    double precision U(N), L(N-1)

    call mxcopyptrtoreal8(U_pr,U,N)

    L = 0d0

    indc=(/ ( Neq*(2*i-1)+1, i=1,H ) /)     ! index of  uic 
    inds=(/ ( Neq*(2*i)+1, i=1,H ) /)       ! index of  uis 

    L(1:Neq)=pL0(U(1:Neq))+U(N-1)*pC1()

    do i=1,H
        L(Neq+2*(i-1)*Neq+1:2*Neq+2*(i-1)*Neq)  = pL0(U(indc(i):indc(i)+Neq-1))
        L(2*Neq+2*(i-1)*Neq+1:3*Neq+2*(i-1)*Neq)= pL0(U(inds(i):inds(i)+Neq-1))
    enddo

! NO PHASE EQUATION

    call mxcopyreal8toptr(L,L_pr,N-1)

    return 
end subroutine CALCUL_L

