!     MEX-file for MATLAB function "L"
!     S. Karkar
!     October 2010

!     The calling arguments must be : (the_object_itself, U, V, H, Neq)

#include "fintrf.h"

subroutine mexFunction(nlhs, plhs, nrhs, prhs)

    use petits_operateurs
    implicit none
    integer nlhs, nrhs, H, Neq_in, N
    mwPointer plhs(nlhs), prhs(nrhs), Q_pr, U_pr, V_pr, mxGetPr, mxCreateDoubleMatrix, H_pr, Neq_pr
    double precision tmp

    if (nlhs /= 1) call mexErrMsgTxt('Error : there must be exactly 1 output argument.')
    if (nrhs /= 5) call mexErrMsgTxt('Error : there must be exactly 5 input arguments.')


!     get the harmonic number H and the equation number Neq
    H_pr = mxGetPr(prhs(4))
    call mxcopyptrtoreal8(H_pr,tmp,1)
    H=int(tmp)

    Neq_pr = mxGetPr(prhs(5))
    call mxcopyptrtoreal8(Neq_pr,tmp,1)
    Neq_in=int(tmp)

    if (Neq_in /= Neq) call mexerrmsgtxt('Neq not consistent with its value in Fortran sources!')

! size of the FREE extended system
    N = Neq*(2*H+1) + 2


! get the U vector pointer
    U_pr = mxGetPr(prhs(2))
    V_pr = mxGetPr(prhs(3))

!     Create matrix for the return argument.
    plhs(1) = mxCreateDoubleMatrix(N-1,1,0)
    Q_pr = mxgetpr(plhs(1))


!     Call the computational subroutine on L with U, H and N values
    call CALCUL_Q(U_pr,V_pr,H,N,Q_pr)


    return
end subroutine mexFunction

subroutine CALCUL_Q(U_pr,V_pr,H,N,Q_pr)
 
    use petits_operateurs
    implicit none
    integer(4) H,N
    real(8) U(N),V(N),Q(N-1),SOM(Neq),SOMC(Neq),SOMS(Neq),pu(Neq),pv(Neq),pu2(Neq),pv2(Neq)
    integer indc(H),inds(H),i,j,ih
    mwPointer U_pr, V_pr, Q_pr

!    interface
!       function pQ(u,v)
!       real(8), intent(in) :: u(:),v(:)
!       real(8) pQ(size(u))
!       end function pQ
!       function pL1(u)
!       real(8), intent(in) :: u(:)
!       real(8) pL1(size(u))
!       end function pL1
!       function pM(u)
!       real(8), intent(in) :: u(:)
!       real(8) pM(size(u))
!       end function pM
!    end interface


call mxcopyptrtoreal8(V_pr,V,N)
    call mxcopyptrtoreal8(U_pr,U,N)



    indc=(/ ( Neq*(2*i-1)+1, i=1,H ) /)     ! index of  uic 
    inds=(/ ( Neq*(2*i)+1, i=1,H ) /)       ! index of  uis 
  
    Q=0.0d0
  
 
  !------------------------------------------------------------------------------
  !     Q(U,V) depends on pQ(.,.)   lambda pL1(.) and  omega pM(.)   see annexe 1
  !------------------------------------------------------------------------------
  
  ! Harmonic zero
    SOM(1:Neq)=0d0
    do j=1,H
      pu  = (/U(indc(j):indc(j)+Neq-1)/)
      pv  = (/V(indc(j):indc(j)+Neq-1)/)
      pu2 = (/U(inds(j):inds(j)+Neq-1)/)
      pv2 = (/V(inds(j):inds(j)+Neq-1)/)

      SOM = SOM + pQ( pu , pv ) + pQ( pu2 , pv2 )

    enddo

    pu = U(1:Neq)
    pv = V(1:Neq)

    Q(1:Neq) = 0.5d0*SOM + pQ(pu,pv) + V(N-1)*pL1(pu)
  
  ! Harmonics 1,2,3, ...
    do ih=1,H
        SOMC(1:Neq)=0d0
        SOMS(1:Neq)=0d0
     
        ! ++++++ 
     
        do j=1,ih-1
          pu  = U(indc(ih-j):indc(ih-j)+Neq-1)
          pv  = V(indc(j):indc(j)+Neq-1)
          pu2 = U(inds(ih-j):inds(ih-j)+Neq-1)
          pv2 = V(inds(j):inds(j)+Neq-1)

          SOMC = SOMC + pQ(pu,pv) - pQ(pu2,pv2)
          SOMS = SOMS + pQ(pu,pv2) + pQ(pu2,pv)

        enddo
     
        ! --------
   
        do j=ih+1,H

          pu  = (/U(indc(j):indc(j)+Neq-1)/)
          pv  = (/V(indc(j-ih):indc(j-ih)+Neq-1)/)
          pu2 = (/U(inds(j):inds(j)+Neq-1)/)
          pv2 = (/V(inds(j-ih):inds(j-ih)+Neq-1)/)

          SOMC = SOMC + pQ(pu,pv) + pQ(pu2,pv2)
          SOMS = SOMS + pQ(pu2,pv) - pQ(pu,pv2)

          pu  = (/U(indc(j-ih):indc(j-ih)+Neq-1)/)
          pv  = (/V(indc(j):indc(j)+Neq-1)/)
          pu2 = (/U(inds(j-ih):inds(j-ih)+Neq-1)/)
          pv2 = (/V(inds(j):inds(j)+Neq-1)/)

          SOMC = SOMC + pQ(pu,pv) + pQ(pu2,pv2)
          SOMS = SOMS -pQ(pu2,pv) + pQ(pu,pv2)
                 
        enddo
      ! 
        Q(indc(ih):indc(ih)+Neq-1)=0.5d0*SOMC + pQ(U(indc(ih):indc(ih)+Neq-1),V(1:Neq))   &           ! Q(U,V)
                                          &   + pQ(U(1:Neq),V(indc(ih):indc(ih)+Neq-1))  &
                                          &   + V(N-1)*pL1(U(indc(ih):indc(ih)+Neq-1)) &   ! lambda.L1(U)
                                          &   - ih*V(N)*pM(U(inds(ih):inds(ih)+Neq-1))   !  - omega.M(U)
                                             
                                             
        Q(inds(ih):inds(ih)+Neq-1)=0.5d0*SOMS + pQ(U(inds(ih):inds(ih)+Neq-1),V(1:Neq))   &          ! Q(U,V)
                                         &    + pQ(U(1:Neq),V(inds(ih):inds(ih)+Neq-1))    &        !
                                         &    + V(N-1)*pL1(U(inds(ih):inds(ih)+Neq-1))  & !  lambda.L1(U)
                                         &    + ih*V(N)*pM(U(indc(ih):indc(ih)+Neq-1))  !  - omega.M(U)
   enddo
 
  
! PHASE EQUATION : NULL

    call mxcopyreal8toptr(Q,Q_pr,N-1)


return 
end subroutine CALCUL_Q

