!     MEX-file / MATLAB function "L" for time-domain
!     orthogonal collocation system continuation of
!     periodic orbits with MANLAB

!     S. Karkar, B. Cochelin, C. Vergez
!     July 2010

!     The calling arguments must be : (the_object_itself, Ninter, Neq)

!     IMPORTANT :   This file is only a header to the global fortran source that
!                   will be generated via the "make" command inside your
!                   class '@XXXX' directory. It WILL NOT compile alone.


subroutine mexFunction(nlhs, plhs, nrhs, prhs)

    
    implicit none
    mwPointer plhs(*), prhs(*)
    mwPointer mxGetPr, mxCreateDoubleMatrix
    mwPointer u_pr, Ninter_pr, Neq_pr, l_pr
    integer(4) nlhs, nrhs, Ninter, Neq, Ntot
    real(8) tmp


!     get the harmonic number H and the equation number Neq
    Ninter_pr = mxGetPr(prhs(3))
    call mxCopyPtrToReal8(Ninter_pr,tmp,1)
    Ninter = int(tmp)

    Neq_pr = mxGetPr(prhs(4))
    call mxCopyPtrToReal8(Neq_pr,tmp,1)
    Neq=int(tmp)

!   total system size
    Ntot = 3*Ninter*Neq + Neq + 2

!     get u pointer pointer

    u_pr = mxGetPr(prhs(2))


!     Create matrix for the return argument.
    plhs(1) = mxCreateDoubleMatrix(Ntot-1,1,0)
!     Load the data into l_pr, which is the output to MATLAB
    l_pr = mxGetPr(plhs(1))


!     Call the computational subroutine with U and V pointers and H, Ninter and Ntot values
    call CALCUL_L(u_pr,l_pr,Ninter,Ntot)


    return
end subroutine mexFunction

subroutine CALCUL_L(u_pr,l_pr,Ninter,Ntot)
 
    use petits_operateurs
    integer(4) Ntot,Ninter,i,j,k,n
    real(8) L(Ntot-1), U(Ntot), pu(Neq), P(4,3), x(3), Y(3*Ninter+1,Neq), lambda
    mwpointer upr, l_pr

    interface
        function pL0(pu)
            use petits_operateurs
            real(8), intent(IN) :: pu(Neq)
            real(8) pL0(Neq)
        end function pL0
        function pC1()
            use petits_operateurs
            real(8) pC1(Neq)
        end function pC1
    end interface

    call mxCopyPtrToReal8(u_pr,U,Ntot)

    L = 0d0

! Definition of the collocation points :
!     x_i (i=1,3) is the ith Gauss point of order 3 on the interval [0,1]
    x=(/ (1-dsqrt(3d0/5d0))/2d0, 1d0/2d0, (1d0+dsqrt(3d0/5d0))/2d0 /)

! Values of the Lagrange polynomials P_k (k=1,4) on the collocation points
! x_i (i=1,3). The polynomial P_k is defined by :
!       P_k(x) = prod[(x-l/3), l=0:3, l!=k] / prod[(k/3-l/3), l=0:3, l!=k]

    P=0d0

    P(1,:) = (x(:)-1d0/3d0)*(x(:)-2d0/3d0)*(x(:)-1) / ((-1d0/3d0)*(-2d0/3d0)*(-1d0))
    P(2,:) = x(:)*(x(:)-2d0/3d0)*(x(:)-1d0) / (1d0/3d0*(-1d0/3d0)*(-2d0/3d0))
    P(3,:) = x(:)*(x(:)-1d0/3d0)*(x(:)-1d0) / (2d0/3d0*1d0/3d0*(-1d0/3d0))
    P(4,:) = x(:)*(x(:)-1d0/3d0)*(x(:)-2d0/3d0) / (1d0*2d0/3d0*1d0/3d0)


    lambda = U(Ntot-1)

! let's reshape the state vector
    Y=0d0

    do n=1,3*Ninter+1
        Y(n,:) = U((n-1)*Neq+1:n*Neq)    ! the nth time sample is a Neq-size column-vector
    enddo

    do j=1,Ninter
        do i=1,3
            pu = (/ (dot_product(Y((j-1)*3+1:(j-1)*3+4,k),P(:,i)),k=1,Neq) /)

            L(((j-1)*3+i-1)*Neq+1:((j-1)*3+i)*Neq) = pL0(pu) + pC1()*lambda
        enddo
    enddo


! Periodicity condition

L(3*Ninter*Neq+1:3*Ninter*Neq+Neq) = U(1:Neq) - U(3*Ninter*Neq+1:3*Ninter*Neq+Neq)



! Phase equation

! Null velocity phase condition : v1(t=0) = 0   ( <=> U(3)=0 )
!!! CAUTION !!!
! The 3rd component must be a velocity-like component that passes through 0 at least once in a period
    L(Ntot-1) = U(3)


    call mxCopyReal8ToPtr(L,l_pr,Ntot-1)

    return 
end subroutine CALCUL_L

