!     MEX-file / MATLAB function "L0" for time-domain
!     orthogonal collocation system continuation of
!     periodic orbits with MANLAB


!     S. Karkar, B. Cochelin, C. Vergez
!     July 2010

!     The calling arguments must be : (the_object_itself, Ninter, Neq)

!     IMPORTANT :   This file is only a header to the global fortran source that
!                   will be generated via the "make" command inside your
!                   class '@XXXX' directory. It WILL NOT compile alone.


subroutine mexFunction(nlhs, plhs, nrhs, prhs)

    implicit none
    mwPointer plhs(*), prhs(*)
    mwPointer mxGetPr, mxCreateDoubleMatrix
    mwPointer u_pr, Ninter_pr, Neq_pr, l0_pr
    integer(4) nlhs, nrhs, Ninter, Neq, Ntot
    real(8) tmp


!     get the numlber of time interval Ninter and the equation number Neq
    Ninter_pr = mxGetPr(prhs(2))
    call mxCopyPtrToReal8(Ninter_pr,tmp,1)
    Ninter = int(tmp)

    Neq_pr = mxGetPr(prhs(3))
    call mxCopyPtrToReal8(Neq_pr,tmp,1)
    Neq=int(tmp)

!   total system size
    Ntot = 3*Ninter*Neq + Neq + 2


!     Create matrix for the return argument.
    plhs(1) = mxCreateDoubleMatrix(Ntot-1,1,0)
!     Load the data into l0_pr, which is the output to MATLAB
    l0_pr = mxGetPr(plhs(1))


!     Call the computational subroutine with U and V pointers and H, Neq and N values
    call CALCUL_L0(l0_pr,Ninter,Ntot)


    return
end subroutine mexFunction

subroutine CALCUL_L0(l0_pr,Ninter,Ntot)
 
    use petits_operateurs
    integer(4) Ninter,Ntot,i
    real(8) L0(Ntot-1)
    mwpointer l0_pr

    interface
        function pC0()
            use petits_operateurs
            real(8) pC0(Neq)
        end function pC0
    end interface

    L0 = 0d0
    
    do i=1,3*Ninter+1    
        L0((i-1)*Neq+1:i*Neq) = pC0()
    enddo

    call mxCopyReal8ToPtr(L0,l0_pr,Ntot-1)

    return 
end subroutine CALCUL_L0

